/**
 * Copyright (c) 2012 Harri Pitkänen <hatapitk@iki.if>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
 * IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ***/

using System;
using System.Windows.Forms;
using System.Text;
using System.Linq;
using System.Reflection;
using System.IO;
using libvoikko;
using System.ComponentModel;

namespace Voikkosample
{
	static class Program
	{
		static void Main()
		{
			try
			{
				VoikkoForm form = new VoikkoForm();
				Application.Run(form);
			}
			catch (VoikkoException e)
			{
				MessageBox.Show(e.Message);
			}
		}
	}
	
	public class VoikkoForm : Form
	{
		private const int INITIAL_WIDTH = 400;
		private TextBox inputArea;
		private Button runButton;
		private Label results;
		private Voikko voikko;
		
		public VoikkoForm()
		{
			voikko = new Voikko("fi", GetDictPath());
			Init();
		}
		
		private string GetDictPath()
		{
			Assembly assembly = Assembly.GetEntryAssembly();
			return Path.GetDirectoryName(assembly.Location);
		}
		
		private void Init()
		{
			SuspendLayout();
			Width = INITIAL_WIDTH + 50;
			Height = 550;
			Text = "Voikko C# sample";
			
			FlowLayoutPanel flp = new FlowLayoutPanel();
			flp.Width = Width;
			flp.Height = Height;
			flp.FlowDirection = FlowDirection.TopDown;
			
			inputArea = new TextBox();
			inputArea.Multiline = true;
			inputArea.ScrollBars = ScrollBars.Vertical;
			inputArea.Width = INITIAL_WIDTH;
			inputArea.Height = 150;
			flp.Controls.Add(inputArea);
			
			runButton = new Button();
			runButton.Text = "Check text";
			runButton.Width = INITIAL_WIDTH;
			runButton.Height = 50;
			flp.Controls.Add(runButton);
			
			{
				Button exitButton = new Button();
				exitButton.Text = "Exit";
				exitButton.Width = INITIAL_WIDTH;
				exitButton.Height = 50;
				flp.Controls.Add(exitButton);
				exitButton.Click += delegate(object sender, EventArgs e) {
					Application.Exit();
				};
			}
			
			results = new Label();
			results.Width = INITIAL_WIDTH;
			results.Height = 250;
			flp.Controls.Add(results);
			
			Controls.Add(flp);
			
			runButton.Click += HandleRunButtonClick;
			
			ResumeLayout(false);
			PerformLayout();
		}

		void HandleRunButtonClick(object sender, EventArgs e)
		{
			string text = inputArea.Text;
			StringBuilder sb = new StringBuilder();
			foreach (Token token in voikko.Tokens(text).Where(t => t.Type == TokenType.WORD))
			{
				string word = token.Text;
				if (voikko.Spell(word))
				{
					sb.AppendLine("Correct: " + word);
				}
				else
				{
					sb.AppendLine("Wrong: " + word);
					foreach (string suggestion in voikko.Suggest(word))
					{
						sb.AppendLine("  Suggestion: " + suggestion);
					}
				}
			}
			results.Text = sb.ToString();
		}
		
		protected override void Dispose(bool disposing)
		{
			if (voikko != null)
			{
				voikko.Dispose();
				voikko = null;
			}
			base.Dispose(disposing);
		}
	}
}
